"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.CryptoUtils = void 0;
const crypto_1 = __importDefault(require("crypto"));
const objects_1 = require("@snickerdoodlelabs/objects");
const argon2_1 = __importDefault(require("argon2"));
const ethers_1 = require("ethers");
const inversify_1 = require("inversify");
const neverthrow_1 = require("neverthrow");
let CryptoUtils = class CryptoUtils {
    constructor() {
        this.cipherAlgorithm = "aes-256-cbc";
    }
    getNonce(nonceSize = 64) {
        const baseString = (0, objects_1.Base64String)(crypto_1.default.randomBytes(nonceSize).toString("base64").slice(0, nonceSize));
        return (0, neverthrow_1.okAsync)(baseString);
    }
    getTokenId() {
        const buf = crypto_1.default.randomBytes(4);
        const hex = buf.toString("hex");
        const bigInt = BigInt(`0x${hex}`);
        return (0, neverthrow_1.okAsync)((0, objects_1.TokenId)(bigInt));
        // This older implementation I'm leaving as a comment. It's very clean, but does not
        // work in a service worker, because the polyfills available do not support randomInt().
        // Ye be warned.
        //return okAsync(TokenId(BigInt(Crypto.randomInt(281474976710655))));
    }
    deriveAESKeyFromSignature(signature, salt) {
        // A signature is a hex string, with 65 bytes. We should convert it to a buffer
        const sourceEntropy = Buffer.from(signature, "hex");
        const saltBuffer = Buffer.from(salt, "hex");
        const keyBuffer = crypto_1.default.pbkdf2Sync(sourceEntropy, saltBuffer, 100, 32, "sha256");
        return (0, neverthrow_1.okAsync)((0, objects_1.AESKey)(keyBuffer.toString("base64")));
    }
    deriveAESKeyFromEVMPrivateKey(evmKey) {
        // We can generate salt by signing a message
        return this.signMessage("PhoebeIsCute", evmKey).map((signature) => {
            // An EVMPrivateKey is a hex string. We should convert it to a buffer
            const sourceEntropy = Buffer.from(evmKey, "hex");
            const saltBuffer = Buffer.from(signature, "hex");
            const keyBuffer = crypto_1.default.pbkdf2Sync(sourceEntropy, saltBuffer, 100, 32, "sha256");
            return (0, objects_1.AESKey)(keyBuffer.toString("base64"));
        });
    }
    deriveCeramicSeedFromEVMPrivateKey(evmKey) {
        return this.signMessage("VarunWasHere", evmKey).map((signature) => {
            // An EVMPrivateKey is a hex string. We should convert it to a buffer
            const sourceEntropy = Buffer.from(evmKey, "hex");
            const saltBuffer = Buffer.from(signature, "hex");
            const keyBuffer = crypto_1.default.pbkdf2Sync(sourceEntropy, saltBuffer, 100, 32, "sha256");
            return new Uint8Array(keyBuffer.buffer);
        });
    }
    createAESKey() {
        return (0, neverthrow_1.okAsync)((0, objects_1.AESKey)(crypto_1.default.randomBytes(32).toString("base64")));
    }
    createEthereumPrivateKey() {
        return (0, neverthrow_1.okAsync)((0, objects_1.EVMPrivateKey)(crypto_1.default.randomBytes(32).toString("hex")));
    }
    getEthereumAccountAddressFromPrivateKey(privateKey) {
        const wallet = new ethers_1.ethers.Wallet(privateKey);
        return (0, objects_1.EVMAccountAddress)(wallet.address);
    }
    verifySignature(message, signature) {
        const address = (0, objects_1.EVMAccountAddress)(ethers_1.ethers.utils.verifyMessage(message, signature));
        return (0, neverthrow_1.okAsync)(address);
    }
    verifyTypedData(domain, types, value, signature) {
        return (0, neverthrow_1.okAsync)((0, objects_1.EVMAccountAddress)(ethers_1.ethers.utils.verifyTypedData(domain, types, value, signature)));
    }
    encryptString(secret, encryptionKey) {
        return this.getNonce(16).map((nonce) => {
            const iv = (0, objects_1.InitializationVector)(nonce);
            try {
                const cipher = crypto_1.default.createCipheriv(this.cipherAlgorithm, Buffer.from(encryptionKey, "base64"), iv);
                // Encrypt the message
                let encryptedData = cipher.update(secret, "utf8", "base64");
                encryptedData += cipher.final("base64");
                return new objects_1.AESEncryptedString((0, objects_1.EncryptedString)(encryptedData), iv);
            }
            catch (e) {
                // This is not ideal error handling, but is better than nothing. At least
                // we will get some logs
                console.error(`Error while encrypting string!`);
                console.error(e);
                return new objects_1.AESEncryptedString((0, objects_1.EncryptedString)("THIS IS AN ERROR"), iv);
            }
        });
    }
    decryptAESEncryptedString(encrypted, encryptionKey) {
        try {
            // The decipher function
            const decipher = crypto_1.default.createDecipheriv(this.cipherAlgorithm, Buffer.from(encryptionKey, "base64"), encrypted.initializationVector);
            // decrypt the message
            let decryptedData = decipher.update(encrypted.data, "base64", "utf8");
            decryptedData += decipher.final("utf8");
            return (0, neverthrow_1.okAsync)(decryptedData);
        }
        catch (e) {
            // This is not ideal error handling, but is better than nothing. At least
            // we will get some logs
            console.error(`Error while deciphering encrypted string!`);
            console.error(e);
            return (0, neverthrow_1.okAsync)("THIS IS AN ERROR");
        }
    }
    // public generateKeyPair(): ResultAsync<void, never> {
    // 	const { publicKey, privateKey } = Crypto.generateKeyPairSync("rsa", {
    // 		// The standard secure default length for RSA keys is 2048 bits
    // 		modulusLength: 2048,
    // 	});
    // 	console.log(publicKey);
    // 	return okAsync(undefined);
    // }
    signMessage(message, privateKey) {
        const wallet = new ethers_1.ethers.Wallet(privateKey);
        return neverthrow_1.ResultAsync.fromSafePromise(wallet.signMessage(message)).map((signature) => {
            return (0, objects_1.Signature)(signature);
        });
    }
    signTypedData(domain, types, value, privateKey) {
        const wallet = new ethers_1.ethers.Wallet(privateKey); // TODO, need to specify default provider (https://github.com/ethers-io/ethers.js/issues/2258)
        return neverthrow_1.ResultAsync.fromSafePromise(wallet._signTypedData(domain, types, value)).map((signature) => {
            return (0, objects_1.Signature)(signature);
        });
    }
    hashStringSHA256(message) {
        const hash = crypto_1.default.createHash("sha256").update(message).digest("base64");
        return (0, neverthrow_1.okAsync)((0, objects_1.SHA256Hash)(hash));
    }
    hashStringArgon2(message) {
        return neverthrow_1.ResultAsync.fromSafePromise(argon2_1.default.hash(message)).map((hash) => {
            return (0, objects_1.Argon2Hash)(hash);
        });
    }
    verifyHashArgon2(hash, message) {
        return neverthrow_1.ResultAsync.fromSafePromise(argon2_1.default.verify(hash, message));
    }
    xmur3(str) {
        let h = 1779033703 ^ str.length;
        for (let i = 0; i < str.length; i++) {
            (h = Math.imul(h ^ str.charCodeAt(i), 3432918353)),
                (h = (h << 13) | (h >>> 19));
        }
        return function () {
            h = Math.imul(h ^ (h >>> 16), 2246822507);
            h = Math.imul(h ^ (h >>> 13), 3266489909);
            return (h ^= h >>> 16) >>> 0;
        };
    }
    sfc32(a, b, c, d) {
        return function () {
            a >>>= 0;
            b >>>= 0;
            c >>>= 0;
            d >>>= 0;
            let t = (a + b) | 0;
            a = b ^ (b >>> 9);
            b = (c + (c << 3)) | 0;
            c = (c << 21) | (c >>> 11);
            d = (d + 1) | 0;
            t = (t + d) | 0;
            c = (c + t) | 0;
            return (t >>> 0) / 4294967296;
        };
    }
    randomInt(randomFunc, low, high) {
        return Math.floor(randomFunc() * (high - low) + low);
    }
    randomBytes(length, seed) {
        // The seed is any string, we can use that to seed the hash method.
        const hash = this.xmur3(seed);
        // Output four 32-bit hashes to provide the seed for sfc32
        const randFunc = this.sfc32(hash(), hash(), hash(), hash());
        const out = new Uint8Array(length);
        for (let i = 0; i < out.length; i++) {
            out[i] = this.randomInt(randFunc, 0, 256);
        }
        return out;
    }
};
CryptoUtils = __decorate([
    (0, inversify_1.injectable)()
], CryptoUtils);
exports.CryptoUtils = CryptoUtils;
//# sourceMappingURL=CryptoUtils.js.map